#! /usr/bin/env python
# -.- coding: utf-8 -.-
#
# Sezen
#
# Copyright © 2009-2010 Seif Lotfy <seif@lotfy.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import time
import gobject
import os
import urllib

def event_exists(uri):
	# TODO: Move this into Zeitgeist's datamodel.py
	return not uri.startswith("file://") or os.path.exists(urllib.unquote(str(uri[7:])))


from xdg.Config import icon_size
from zeitgeist.client import ZeitgeistClient
from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation, \
	ResultType, TimeRange

try:
	CLIENT = ZeitgeistClient()
except RuntimeError, e:
	print "Unable to connect to Zeitgeist: %s" % e
	CLIENT = None
	
import dbus
try:
	_bus = dbus.SessionBus()
	_fts = _bus.get_object('org.gnome.zeitgeist.Engine',
	                      '/org/gnome/zeitgeist/index/activity')
	fts = dbus.Interface(_fts, 'org.gnome.zeitgeist.Index')
except:
	print "Unable to connect to Zeitgeist FTS: %s" % e




SUPPORTED_SOURCES = {
	# TODO: Please change this description stuff. To give a single reason,
	# saying "videos watched" is a lie because you may have edited them, or
	# it may just be a "bookmark event".
	#
	# Also, how this is implemented now won't work fine with i18n.	
	#"sezen://home": {"name": "Home", "icon":"gtk-home", "uri": "sezen://home"},
	
	Interpretation.AUDIO.uri: {"name": "Audio", "icon":"gnome-mime-audio", "uri": Interpretation.AUDIO},
	Interpretation.DOCUMENT.uri: {"name": "Documents", "icon":"x-office-document", "uri": Interpretation.DOCUMENT},
	Interpretation.IMAGE.uri: {"name": "Images", "icon":"image", "uri": Interpretation.IMAGE},
	Interpretation.VIDEO.uri: {"name": "Video", "icon":"gnome-mime-video", "uri": Interpretation.VIDEO},
	#Interpretation.WEBSITE.uri: {"name": "Websites", "icon":"gnome-globe", "uri": Interpretation.WEBSITE},
	Interpretation.IMMESSAGE.uri: {"name": "People", "uri":Interpretation.IMMESSAGE, "icon":"stock_person"},
	#Interpretation.SOURCE_CODE.uri: {"name": "Development", "icon":"stock_new-presentation", "uri": Interpretation.SOURCE_CODE},
}

for parent in SUPPORTED_SOURCES.values():
	parent = parent["uri"]
	for child in parent.get_children():
		print child

SUPPORTED_SOURCES["zeitgeist://other"] = {"name": "Other", "icon":"applications-other", "uri": "z://other"}

def match_interpretation(interpretation):
	if interpretation == Interpretation.MUSIC_PIECE:
		return Interpretation.AUDIO
	elif interpretation == Interpretation.SOURCE_CODE:
		return Interpretation.DOCUMENT
	else:
		return interpretation
	
class DataSink(gobject.GObject):
	__gsignals__ = {
		"reload" : (gobject.SIGNAL_RUN_FIRST,
					gobject.TYPE_NONE,())}

	def __init__(self):
		gobject.GObject.__init__(self)
		self.zg = CLIENT
		self.events = []
		self.last_event = None
		self.categories = {}
		for source in SUPPORTED_SOURCES.keys():
			self.categories[source] = []
		self.categories["other"] = []
		obj = dbus.SessionBus().get_object("org.gnome.zeitgeist.Engine",
			"/org/gnome/zeitgeist/geolocation")
		self.geoloc = dbus.Interface(obj, "org.gnome.zeitgeist.Log")
		
		def notify_insert_handler(time_range, events):
			if self.last_event != events[0].subjects[0].uri:
				self.last_event = events[0].subjects[0].uri
				self.emit("reload")
		
		events = []
		self.zg.install_monitor(TimeRange.always(), events,
			notify_insert_handler, notify_insert_handler)
		
	def get_sources(self):
		return SUPPORTED_SOURCES

	def get_events(self, location):
		result = self.geoloc.FindEventsForLocations(location, (0, 10000000000000), [], 2, 0, 2)		
		events = []
		for r in result:
			events.append(r[:len(r)-1])
		events = map(Event, events)
		
		self._categorise_events(events)
		return self.categories
		
	def _categorise_events(self, events):
		self.categories.clear()
		for source in SUPPORTED_SOURCES.keys():
			self.categories[source] = []
		for event in events:
			if event_exists(event.subjects[0].uri):
				subject = event.subjects[0]
				subject.interpretation = match_interpretation(subject.interpretation)

				if not self.categories.has_key(subject.interpretation):
					self.categories["zeitgeist://other"].append(event)
				else:
					self.categories[subject.interpretation].append(event)
			
	
datasink = DataSink()
