#!/usr/bin/env python
# -*- coding: utf-8 -*-
#!/usr/bin/env python
# -*- coding: utf-8 -*-

import gi

from gi.repository import GtkClutter
GtkClutter.init([])
from gi.repository import GObject, Gtk, GtkChamplain, Gio

GObject.threads_init()
GtkClutter.init([])

from gi.repository import GdkPixbuf
from gi.repository import Pango
from gi.repository import Clutter
from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation
from gi.repository import Champlain
from datasink import SUPPORTED_SOURCES, datasink, match_interpretation
from ui_utils import icon_factory, no_pixbuf, get_icon

from markers import create_marker_layer

class LauncherGTK:
    def __init__(self):
        self.window = Gtk.Window()
        self.window.set_border_width(0)
        self.window.set_title("El Loco")
        self.window.connect("destroy", Gtk.main_quit)
        
        vbox = Gtk.VBox(homogeneous=False, spacing=3)
        hbox = Gtk.HPaned()
        
        self.sidebar = Sidebar()

        embed = GtkChamplain.Embed()
        embed.set_size_request(640, 480)

        self.view = embed.get_view()
        self.view.set_reactive(True)
        self.view.connect('button-press-event', self.mouse_click_cb, self.view)

        #self.view.set_property('scroll-mode', int(Champlain.ScrollMode.KINETIC))
        self.view.set_property('zoom-level', 5)
        #self.view.set_property('license-text', "Don't eat cereals with orange juice\nIt tastes bad")
        #self.view.set_property('show-scale', True)
        
        self.radius =  1.0/5
        
        self.latitude = -1
        self.longitude = -1
        

        self.layer = create_marker_layer(self.view)
        self.view.add_layer(self.layer)
        self.layer.hide_all_markers()
        
        
        #self.polygon = polygon = Champlain.Polygon()
        # Cheap approx of Highway 10
       
        #polygon.set_stroke_width(5.0);
        #polygon.set_property("mark-points", True)
        #self.view.add_polygon(polygon)

        bbox = Gtk.HBox(homogeneous=False, spacing=6)
        button = Gtk.Button(stock=Gtk.STOCK_ZOOM_IN)
        button.connect("clicked", self.zoom_in)
        bbox.add(button)

        button = Gtk.Button(stock=Gtk.STOCK_ZOOM_OUT)
        button.connect("clicked", self.zoom_out)
        bbox.add(button)

        button = Gtk.ToggleButton(label="Markers")
        button.set_active(False)
        button.connect("toggled", self.toggle_layer)
        #bbox.add(button)

        combo = Gtk.ComboBoxText()
        self.map_source_factory = Champlain.MapSourceFactory.dup_default()
        liststore = Gtk.ListStore(str, str)
        for source in self.map_source_factory.get_registered():
            liststore.append([source.get_id(), source.get_name()])
            combo.append_text(source.get_name())
        combo.set_model(liststore)
        # TODO: Not sure what this is
        #combo.set_attributes(combo.get_cells()[0], text=1)
        combo.set_active(0)
        combo.connect("changed", self.map_source_changed)
        bbox.add(combo)

        self.spinbutton = Gtk.SpinButton(adjustment=Gtk.Adjustment(lower=0, upper=20, 
            value=1, step_increment=1))
        self.spinbutton.connect("changed", self.zoom_changed)
        self.view.connect("notify::zoom-level", self.map_zoom_changed)
        self.spinbutton.set_value(5)
        bbox.add(self.spinbutton)

        button = Gtk.Image()
        self.view.connect("notify::state", self.view_state_changed, button)
        bbox.pack_end(button, False, False, 0)        

        vbox.pack_start(bbox, False, False, 0)
        vbox.add(embed)

        hbox.pack1(vbox, True, True)
        
        hbox.pack2(self.sidebar, False, False)
        self.window.add(hbox)
        datasink.connect("reload", self.reload)


        self.window.show_all()

    def reload(self, widget):
        zoom = self.view.get_zoom_level()
        self.sidebar.set_location((self.longitude, self.latitude, 2.0/zoom))


    def zoom_in(self, widget):
        self.view.zoom_in()

    def zoom_out(self, widget):
        self.view.zoom_out()

    def toggle_layer(self, widget):
        if widget.get_active():
            self.polygon.show()    
            self.layer.animate_in_all_markers()
        else:
            self.polygon.hide()    
            self.layer.animate_out_all_markers()

    def mouse_click_cb(self, actor, event, view):
        #(result, lat, lon) = view.get_coords_from_event(event)
        lon, lat = event.get_coords()

        self.latitude = view.y_to_latitude(lat)
        self.longitude = view.x_to_longitude(lon)
        print "Mouse click at: %f %f" % (self.longitude, self.latitude)
        
#        self.polygon.clear_points()
#        self.polygon.append_point(lat, lon)
#        self.polygon.show_all()
        
        zoom = self.view.get_zoom_level()
        self.sidebar.set_location((self.longitude, self.latitude, 2.0/zoom))
        
    def zoom_changed(self, widget):
        self.view.set_property("zoom-level", self.spinbutton.get_value_as_int())
        
        if self.latitude and self.longitude:
            zoom = self.view.get_zoom_level()
            self.sidebar.set_location((self.longitude, self.latitude, 2.0/zoom))

    def map_source_changed(self, widget):
        model = widget.get_model()
        iter = widget.get_active_iter()
        id = model.get_value(iter, 0)
        source = self.map_source_factory.create(id);
        self.view.set_property("map-source", source)

    def map_zoom_changed(self, widget, value):
        self.spinbutton.set_value(self.view.get_property("zoom-level"))

    def view_state_changed(self, view, paramspec, image):
        state = view.get_property("state")
        if int(state) == int(Champlain.State.LOADING):
            image.set_from_stock(Gtk.STOCK_NETWORK, Gtk.IconSize.BUTTON)
        else:
            image.clear()
            


class Sidebar(Gtk.HBox):
    def __init__(self):
        Gtk.HBox.__init__(self)
        self.label = Gtk.Label()
        self.label.set_markup("<b>Location Activities</b>")
        self.location = (-1, -1, 1)
        self.iconview = TreeView()
        self.notebook = Gtk.Notebook()
        self.set_size_request(200,-1)
        vbox = Gtk.VBox()
        self.notebook.set_show_tabs(False)
        vbox.set_spacing(12)
        vbox.pack_start(self.label, False, False, 0)
        vbox.pack_start(self.iconview, True, True, 0)
        self.notebook.append_page(vbox, Gtk.Box())
        self.pack_start(self.notebook, True, True, 0)
        #self.notebook.set_property("tab-fill", 9)
        #self.notebook.set_tab_label_packing(vbox, True, True, Gtk.PackType.START)
        
    def set_location(self, location):
        self.location = location
        events = datasink.get_events(location)
        self.iconview.set_categories(events)
          

class TreeView(Gtk.ScrolledWindow):
    def __init__(self):
        Gtk.ScrolledWindow.__init__(self)
        self.set_policy(2 ,1)
        self.tree_store = Gtk.TreeStore(GdkPixbuf.Pixbuf, str, GObject.TYPE_PYOBJECT)
        self.tree_view = Gtk.TreeView()
        self.tree_view.set_model(self.tree_store)
        self.selection = self.tree_view.get_selection()
        self.selection.set_mode(Gtk.SelectionMode.BROWSE)
        self.tree_view.set_headers_visible(False)
        cell = Gtk.CellRendererPixbuf()
        column = Gtk.TreeViewColumn()
        column.pack_start(cell, False)
        column.add_attribute(cell, "icon", 0)
        #self.tree_view.append_column(column)
        
        cell = Gtk.CellRendererText()
        cell.set_property('ellipsize', 2)
        column = Gtk.TreeViewColumn()
        column.pack_start(cell, False)
        column.add_attribute(cell, "markup", 1)
        self.tree_view.append_column(column)

        self.tree_view.collapse_all()
        self.add_with_viewport(self.tree_view)
        self.iters = {}
        self.show()
        
    def set_categories(self, events):
       self.clear()
       keys = events.keys()
       keys.sort()
       for key in keys:
           if not len(events[key]) == 0:
               text = "<b>%s</b>"%SUPPORTED_SOURCES[key]["name"]
               self.iters[key] = self.tree_store.append(None, [None, text, None])
       for event_list in events.values():
           for event in event_list:
               self.add_item(event)
       self.tree_view.expand_all()
       self.show()
        
    #def launch(self, view, path, w):
    #    item = self.model[path][2]
    #    launch.launch_uri(item.uri)
        
    def clear(self):
        self.iters.clear()
        self.tree_store.clear()
        
        
    def add_item(self, item):
        try:
            self.tree_store.append(self.iters[item.subjects[0].interpretation], [None, item.subjects[0].text, None])
        except:
            self.tree_store.append(self.iters["zeitgeist://other"], [None, item.subjects[0].text, None])

if __name__ == "__main__":
    import sys
    #GObject.threads_init()
    Clutter.init(sys.argv)
    LauncherGTK()
    Gtk.main()
