#! /usr/bin/env python
# -.- coding: utf-8 -.-
#
# Sezen
#
# Copyright © 2009-2010 Seif Lotfy <seif@lotfy.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path
from gi.repository import GObject
import urllib
import os
from gi.repository import Gio as gio
from gi.repository import GLib
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GdkPixbuf
#import gnome.ui
from gi.repository import Pango
import datetime


from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation
from xdg.Config import icon_size

pixbuf_cache = {}
no_pixbuf = GdkPixbuf.Pixbuf()

def event_exists(uri):
	# TODO: Move this into Zeitgeist's datamodel.py
	return not uri.startswith("file://") or os.path.exists(urllib.unquote(str(uri[7:])))

def get_pixbuf_from_cache(gicon, callback, size=24):
	name = gicon.to_string()
	if name in pixbuf_cache:
		callback(pixbuf_cache[name])
	else:
		loadable = gicon.__gtype__.is_a(gio.LoadableIcon)
		if loadable:
			# load asynchronously
			def read_finished(input_stream, res):
				data = input_stream.read_finish(res)
				input_stream.close_async(lambda s, r: s.close_finish(r))

				def idle_cb():
					loader = Gdk.PixbufLoader()
					loader.write(data)
					loader.close()
					pixbuf = loader.get_pixbuf()
					pixbuf_cache[name] = pixbuf
					callback(pixbuf)
					return False

				glib.idle_add(idle_cb)

			def load_finished(arg1, res):
				stream, icon_type = gicon.load_finish(res)
				def idle_cb():
					data = stream.read_async(100*1024, read_finished, glib.PRIORITY_LOW)
					return False

				# we need to finish in idle callback, otherwise this thread
				# will hold global lock and we won't get into main loop
				glib.idle_add(idle_cb)

			gicon.load_async(load_finished)
		else:
			theme = Gtk.IconTheme.get_default()
			# blocks: do something with it
			gtk_icon = theme.lookup_by_gicon(gicon, size, size)
			if gtk_icon: icon = gtk_icon.load_icon()
			else: icon = None
			pixbuf_cache[name] = icon
			callback(icon)


def get_icon(item, callback):
	callback(None)
	uri = item.uri
	mimetype = item.mimetype
	size = 16
	f = gio.file_new_for_uri(uri)
	def callback2( arg1, arg2):
		info = f.query_info_finish(arg2)
		icon_path = info.get_attribute_byte_string(gio.FILE_ATTRIBUTE_THUMBNAIL_PATH)

		def pixbuf_loaded_cb(pixbuf):
			if pixbuf is None: return
			icon = pixbuf.scale_simple(size * pixbuf.get_width() / pixbuf.get_height(),
											size, GdkPixbuf.InterpType.BILINEAR)
			callback(icon)

		def add_frame(pixbuf):
			pixbuf_loaded_cb(icon_factory.make_icon_frame(pixbuf, 16))
			
		#print "***", icon
		"""
		if icon_path:
			gicon = gio.FileIcon(gio.File(icon_path))
			get_pixbuf_from_cache(gicon, add_frame, 16)
		else:
			gicon = info.get_icon()
			get_pixbuf_from_cache(gicon, pixbuf_loaded_cb)
		"""
		icon = f.icon_new()
	
	if f.is_native():
		#attr= gio.FILE_ATTRIBUTE_STANDARD_ICON + "," +	gio.FILE_ATTRIBUTE_THUMBNAIL_PATH
		#f.query_info_async(attr, callback2)
		icon = f.icon_new()
	else:
		#gicon = gio.content_type_get_icon(mimetype)
		#icon = get_pixbuf_from_cache(gicon, lambda x: callback(x))
		icon = f.icon_new()

def shade_gdk_color(color, shade):
	"""
	Shades a color by a fraction

	Arguments:
	:param color: a gdk color
	:param shade: fraction by which to shade the color

	:returns: a :class:`Color <gtk.gdk.Color>`
	"""
	f = lambda num: min((num * shade, 65535.0))
	if gtk.pygtk_version >= (2, 16, 0):
		color.red = f(color.red)
		color.green = f(color.green)
		color.blue = f(color.blue)
	else:
		red = int(f(color.red))
		green = int(f(color.green))
		blue = int(f(color.blue))
		color = gtk.gdk.Color(red=red, green=green, blue=blue)
	return color

def match_interpretation(interpretation):
	if interpretation == Interpretation.MUSIC_PIECE:
		return Interpretation.AUDIO
	elif interpretation == Interpretation.SOURCE_CODE:
		return Interpretation.DOCUMENT
	else:
		return interpretation


class IconFactory:
    '''
    Icon lookup swiss-army knife (from menutreemodel.py)
    '''
    CACHE = {}

    def load_icon_from_path(self, icon_path, icon_size = None):
        if os.path.isfile(icon_path):
            try:
                if icon_size:
                    # constrain height, not width
                    return gtk.gdk.pixbuf_new_from_file_at_size(icon_path, -1, int(icon_size))
                else:
                    return gtk.gdk.pixbuf_new_from_file(icon_path)
            except:
                pass
        return None

    def load_icon_from_data_dirs(self, icon_value, icon_size = None):
        data_dirs = None
        if os.environ.has_key("XDG_DATA_DIRS"):
            data_dirs = os.environ["XDG_DATA_DIRS"]
        if not data_dirs:
            data_dirs = "/usr/local/share/:/usr/share/"

        for data_dir in data_dirs.split(":"):
            retval = self.load_icon_from_path(os.path.join(data_dir, "pixmaps", icon_value),
                                              icon_size)
            if retval:
                return retval
            
            retval = self.load_icon_from_path(os.path.join(data_dir, "icons", icon_value),
                                              icon_size)
            if retval:
                return retval

        return None

    def scale_to_bounded(self, icon, size):
        if icon:
            if icon.get_height() > size:
                return icon.scale_simple(size * icon.get_width() / icon.get_height(),
                                         size,
                                         gtk.gdk.INTERP_BILINEAR)
            else:
                return icon
        else:
            return None

    def load_icon(self, icon_value, icon_size, force_size = True):
        assert icon_value, "No icon to load!"
        if self.CACHE.has_key(icon_value+str(icon_size)):
            return self.CACHE[icon_value+str(icon_size)]
            
        if isinstance(icon_size, Gtk.IconSize):
            icon_size = gtk.icon_size_lookup(icon_size)[0]
            force_size = True

        if isinstance(icon_value, GdkPixbuf.Pixbuf):
            if force_size:
                return self.scale_to_bounded(icon_value, icon_size)
            return icon_value

        if os.path.isabs(icon_value):
            icon = self.load_icon_from_path(icon_value, icon_size)
            if icon:
                if force_size:
                    return self.scale_to_bounded(icon, icon_size)
                return icon
            icon_name = os.path.basename(icon_value)
        else:
            icon_name = icon_value
    
        if icon_name.endswith(".png"):
            icon_name = icon_name[:-len(".png")]
        elif icon_name.endswith(".xpm"):
            icon_name = icon_name[:-len(".xpm")]
        elif icon_name.endswith(".svg"):
            icon_name = icon_name[:-len(".svg")]
    
        icon = None
        info = icon_theme.lookup_icon(icon_name, icon_size, Gtk.IconLookupFlags.USE_BUILTIN)
        if info:
        
            if icon_name.startswith("gtk-"):
                # NOTE: IconInfo/IconTheme.load_icon leaks a ref to the icon, so
                #       load it manually.
                # NOTE: The bindings are also broken for Gtk's builtin pixbufs:
                #       IconInfo.get_builtin_pixbuf always returns None.
                icon = info.load_icon()
            elif info.get_filename():
                icon = self.load_icon_from_path(info.get_filename())
        else:
            icon = self.load_icon_from_data_dirs(icon_value, icon_size) # Fallback

        if icon and force_size:
            icon = self.scale_to_bounded(icon, icon_size)
        self.CACHE[icon_value+str(icon_size)] = icon
        return icon

    def load_image(self, icon_value, icon_size, force_size = True):
        pixbuf = self.load_icon(icon_value, icon_size, force_size)
        img = gtk.Image()
        img.set_from_pixbuf(pixbuf)
        img.show()
        return img

    def make_icon_frame(self, thumb, icon_size = None, blend = False):
        border = 1

        mythumb = GdkPixbuf.Pixbuf(thumb.get_colorspace(),
                                 True,
                                 thumb.get_bits_per_sample(),
                                 thumb.get_width(),
                                 thumb.get_height())
        mythumb.fill(0x00000080) # black, 50% transparent
        if blend:
            thumb.composite(mythumb, 0, 0,
                            thumb.get_width(), thumb.get_height(),
                            0, 0,
                            1.0, 1.0,
                            gtk.gdk.INTERP_NEAREST,
                            128)
        thumb.copy_area(border, border,
                        thumb.get_width() - (border * 2), thumb.get_height() - (border * 2),
                        mythumb,
                        border, border)
        return mythumb

    def transparentize(self, pixbuf, percent):
        pixbuf = pixbuf.add_alpha(False, '0', '0', '0')
        for row in pixbuf.get_pixels_array():
            for pix in row:
                pix[3] = min(int(pix[3]), 255 - (percent * 0.01 * 255))
        return pixbuf

    def colorshift(self, pixbuf, shift):
        pixbuf = pixbuf.copy()
        for row in pixbuf.get_pixels_array():
            for pix in row:
                pix[0] = min(255, int(pix[0]) + shift)
                pix[1] = min(255, int(pix[1]) + shift)
                pix[2] = min(255, int(pix[2]) + shift)
        return pixbuf

    def greyscale(self, pixbuf):
        pixbuf = pixbuf.copy()
        for row in pixbuf.get_pixels_array():
            for pix in row:
                pix[0] = pix[1] = pix[2] = (int(pix[0]) + int(pix[1]) + int(pix[2])) / 3
        return pixbuf


class Thumbnailer:
    def __init__(self):
        self.cached_icon = {}

    def get_icon(self, uri, mimetype, icon_size, timestamp = 0):
        self.uri = uri
        self.mimetype = mimetype
        if not self.cached_icon.has_key(uri+mimetype+str(icon_size)):
            self.cached_icon[uri+mimetype+str(icon_size)] = self._lookup_or_make_thumb(icon_size, timestamp)
        return self.cached_icon[uri+mimetype+str(icon_size)]

    def _lookup_or_make_thumb(self, icon_size, timestamp):
        ### FIXME: Trash URIs crash older libgnomeui's icon lookup, so we avoid it
        if self.uri.startswith("trash:"):
            return self._get_trash_icon()
        
        icon_name, icon_type = \
                   gnome.ui.icon_lookup(icon_theme, thumb_factory, self.uri, self.mimetype, 0)
        try:
            if icon_type == gnome.ui.ICON_LOOKUP_RESULT_FLAGS_THUMBNAIL or \
                   thumb_factory.has_valid_failed_thumbnail(self.uri, timestamp):
                # Use existing thumbnail
                thumb = icon_factory.load_icon(icon_name, icon_size)
            elif self._is_local_uri(self.uri):
                # Generate a thumbnail for local files only
                print " *** Calling generate_thumbnail for", self.uri
                thumb = thumb_factory.generate_thumbnail(self.uri, self.mimetype)
                thumb_factory.save_thumbnail(thumb, self.uri, timestamp)

            if thumb:
                # Fixup the thumbnail a bit
                thumb = self._nicer_dimensions(thumb)
                thumb = icon_factory.make_icon_frame(thumb, icon_size)

                return thumb
        except:
            pass

        # Fallback to mime-type icon on failure
        return icon_factory.load_icon(icon_name, icon_size)

    def _get_trash_icon(self, uri):
        trash_icon = "gnome-fs-trash-full"
        import gimmie_trash
        if gimmie_trash.trash_monitor.is_empty():
            trash_icon = "gnome-fs-trash-empty"
        return icon_factory.load_icon(trash_icon, icon_size)

    def _is_local_uri(self, uri):
        # NOTE: gnomevfs.URI.is_local seems to hang for some URIs (e.g. ssh
        #       or http).  So look in a list of local schemes which comes
        #       directly from gnome_vfs_uri_is_local_scheme.
        scheme, path = urllib.splittype(self.get_uri() or "")
        return not scheme or scheme in ("file", "help", "ghelp", "gnome-help", "trash",
                                        "man", "info", "hardware", "search", "pipe",
                                        "gnome-trash")

    def _nicer_dimensions(self, icon):
        ### Constrain thumb dimensions to 1:1.2
        if float(icon.get_height()) / float(icon.get_width()) > 1.3:
            return icon.subpixbuf(0, 0,
                                  icon.get_width(), int(icon.get_width() * 1.3))
        return icon



def get_category_icon(icon, size=24):
    try:
        return icon_theme.load_icon (icon, size, 0)
    except:
        return None



icon_factory = IconFactory()
icon_theme = Gtk.IconTheme.get_default()
#Not introspected
#thumb_factory = gnome.ui.ThumbnailFactory("normal")
#thumbnailer = Thumbnailer()

ITEM_CACHE = {}



class LaunchManager:
    '''
    A program lauching utility which handles opening a URI or executing a
    program or .desktop launcher, handling variable expansion in the Exec
    string.

    Adds the launched URI or launcher to the ~/.recently-used log.  Sets a
    DESKTOP_STARTUP_ID environment variable containing useful information such
    as the URI which caused the program execution and a timestamp.

    See the startup notification spec for more information on
    DESKTOP_STARTUP_IDs.
    '''
    
    def __init__(self):
        self.recent_model = None

    def _get_recent_model(self):
        # FIXME: This avoids import cycles
        if not self.recent_model:
            import gimmie_recent
            self.recent_model = gimmie_recent.recent_model
        return self.recent_model

    def launch_uri(self, uri, mimetype = None):
        assert uri, "Must specify URI to launch"
        
        child = os.fork()
        if not child:
            # Inside forked child
            os.setsid()
            os.environ['GIMMIE_LAUNCHER'] = uri
            os.environ['DESKTOP_STARTUP_ID'] = self.make_startup_id(uri)
            os.spawnlp(os.P_NOWAIT, "gnome-open", "gnome-open", uri)
            os._exit(0)
        else:
            os.wait()

            if not mimetype:
                mimetype = "application/octet-stream"
                try:
                    # Use XDG to lookup mime type based on file name.
                    # gtk_recent_manager_add_full requires it.
                    import xdg.Mime
                    mimetype = xdg.Mime.get_type_by_name(uri)
                    if mimetype:
                        mimetype = str(mimetype)
                    return mimetype
                except (ImportError, NameError):
                    print " !!! No mimetype found for URI: %s" % uri

            self._get_recent_model().add(uri=uri, mimetype=mimetype)
        return child

    def get_local_path(self, uri):
        scheme, path = urllib.splittype(uri)
        if scheme == None:
            return uri
        elif scheme == "file":
            path = urllib.url2pathname(path)
            if path[:3] == "///":
                path = path[2:]
            return path
        return None

    def launch_command_with_uris(self, command, uri_list, launcher_uri = None):
        if command.rfind("%U") > -1:
            uri_str = ""
            for uri in uri_list:
                uri_str = uri_str + " " + uri
            return self.launch_command(command.replace("%U", uri_str), launcher_uri)
        elif command.rfind("%F") > -1:
            file_str = ""
            for uri in uri_list:
                uri = self.get_local_path(self, uri)
                if uri:
                    file_str = file_str + " " + uri
                else:
                    print " !!! Command does not support non-file URLs: ", command
            return self.launch_command(command.replace("%F", file_str), launcher_uri)
        elif command.rfind("%u") > -1:
            startup_ids = []
            for uri in uri_list:
                startup_ids.append(self.launch_command(command.replace("%u", uri), launcher_uri))
            else:
                return self.launch_command(command.replace("%u", ""), launcher_uri)
            return startup_ids
        elif command.rfind("%f") > -1:
            startup_ids = []
            for uri in uri_list:
                uri = self.get_local_path(self, uri)
                if uri:
                    startup_ids.append(self.launch_command(command.replace("%f", uri),
                                                           launcher_uri))
                else:
                    print " !!! Command does not support non-file URLs: ", command
            else:
                return self.launch_command(command.replace("%f", ""), launcher_uri)
            return startup_ids
        else:
            return self.launch_command(command, launcher_uri)

    def make_startup_id(self, key, ev_time = None):
        if not ev_time:
            ev_time = gtk.get_current_event_time()
        if not key:
            return "GIMMIE_TIME%d" % ev_time
        else:
            return "GIMMIE:%s_TIME%d" % (key, ev_time)

    def parse_startup_id(self, id):
        if id and id.startswith("GIMMIE:"):
            try:
                uri = id[len("GIMMIE:"):id.rfind("_TIME")]
                timestamp = id[id.rfind("_TIME") + len("_TIME"):]
                return (uri, timestamp)
            except IndexError:
                pass
        return (None, None)

    def launch_command(self, command, launcher_uri = None):
        startup_id = self.make_startup_id(launcher_uri)
        child = os.fork()
        if not child:
            # Inside forked child
            os.setsid()
            os.environ['DESKTOP_STARTUP_ID'] = startup_id
            if launcher_uri:
                os.environ['GIMMIE_LAUNCHER'] = launcher_uri
            os.popen2(command)
            os._exit(0)
        else:
            os.wait()
            if launcher_uri:
                self._get_recent_model().add(uri=launcher_uri,
                                            mimetype="application/x-desktop",
                                            groups=["Launchers"])
            return (child, startup_id)
           
launch = LaunchManager()
