#! /usr/bin/env python
# -.- coding: utf-8 -.-
#
# Sezen
#
# Copyright © 2009-2010 Seif Lotfy <seif@lotfy.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gobject
import glib
import pango
import gtk
import commands
import time
import datetime
from ui_utils import *
from model.datasink import SUPPORTED_SOURCES, DataSink
import gio
import datetime

datasink = DataSink()
datasink.load_events()

class MenuBar(gtk.MenuBar):
	def __init__(self):
		gtk.MenuBar.__init__(self)
		keys = SUPPORTED_SOURCES.keys()
		keys.sort()
		for source in keys:
			menu_item = MenuItem(SUPPORTED_SOURCES[source])
			menu_item.clear()
			self.append(menu_item)
		datasink.connect("reload", self.load)
		
	def load(self, widgets):
		for menu_item in self:
			menu_item.clear()
			menu_item._set_timeperiods()

class MenuItem(gtk.ImageMenuItem):
	
	def __init__(self, category):
		gtk.ImageMenuItem.__init__(self, category["name"])
		self.time_menu = gtk.Menu()
		self.source = category
		self._set_timeperiods()
		self.set_submenu(self.time_menu)
		icon = icon_factory.load_icon(category["icon"], 16, True)
		image = gtk.image_new_from_pixbuf(icon)
		
		self.set_image(image)
		self.show()
		
	def clear(self):
		for item in self.time_menu:
			self.time_menu.remove(item)
		
	def _set_timeperiods(self):
		
		for event in datasink.categories[self.source["uri"]][0:7]:
			self.add_item(event, self.time_menu)
		if len(datasink.categories[self.source["uri"]]) > 7:
			mi = gtk.MenuItem("More...")
			self.time_menu.append(gtk.SeparatorMenuItem ())
			self.time_menu.append(mi)
			mi.connect("activate", lambda x: commands.getstatusoutput("/home/seif/Projects/sezen-vala/src/sezen"))
		
		self.show_all()
		
	def add_item(self, event, menu):
		mi = Item(event)
		menu.append(mi)

class Item(gtk.ImageMenuItem):
	def __init__(self, event):
		gtk.ImageMenuItem.__init__(self)
		self.event = event
		label = gtk.Label(event.subjects[0].text)
		label.set_alignment(0.0, 0.5)
		image = gtk.Image()
		label.set_ellipsize(pango.ELLIPSIZE_END)
		self.set_size_request(200, -1)
		self.add(label)
		item = event.subjects[0]
		
		def callback(icon):
			icon = icon if icon is not None else no_pixbuf
			image.set_from_pixbuf(icon)
			self.set_image(image)
			self.show_all()
		get_icon(item, callback)
		
		self.connect("activate", self.launch_uri)
		
	def launch_uri(self, widget):
		launch.launch_uri(self.event.subjects[0].uri)
		
